/* boards_list.c
 * Linux CAN-bus device driver.
 * Written for new CAN driver version by Pavel Pisa - OCERA team member
 * email:pisa@cmp.felk.cvut.cz
 * This software is released under the GPL-License.
 * Version lincan-0.2  9 Jul 2003
 */

#include "../include/can.h"
#include "../include/can_sysdep.h"
#include "../include/main.h"

#include ".supported_cards.h"

#include "../include/pip.h"
#include "../include/pccan.h"
#include "../include/smartcan.h"
#include "../include/pc-i03.h"
#include "../include/pcm3680.h"
#include "../include/m437.h"
#include "../include/template.h"
#include "../include/i82527.h"
#include "../include/aim104.h"
#include "../include/pcccan.h"

extern int template_register(struct hwspecops_t *hwspecops);
extern int pip5_register(struct hwspecops_t *hwspecops);
extern int pip6_register(struct hwspecops_t *hwspecops);
extern int smartcan_register(struct hwspecops_t *hwspecops);
extern int pccanf_register(struct hwspecops_t *hwspecops);
extern int pccand_register(struct hwspecops_t *hwspecops);
extern int pccanq_register(struct hwspecops_t *hwspecops);
extern int nsi_register(struct hwspecops_t *hwspecops);
extern int cc104_register(struct hwspecops_t *hwspecops);
extern int pci03_register(struct hwspecops_t *hwspecops);
extern int pcm3680_register(struct hwspecops_t *hwspecops);
extern int aim104_register(struct hwspecops_t *hwspecops);
extern int pcccan_register(struct hwspecops_t *hwspecops);
extern int ssv_register(struct hwspecops_t *hwspecops);
extern int bfadcan_register(struct hwspecops_t *hwspecops);
extern int pikronisa_register(struct hwspecops_t *hwspecops);
extern int virtual_register(struct hwspecops_t *hwspecops);

const struct boardtype_t can_boardtypes[]={
    #ifdef ENABLE_CARD_template
	{"template", template_register, 1},
    #endif
    #ifdef ENABLE_CARD_template
	{"virtual", virtual_register, 0},
    #endif
    #ifdef ENABLE_CARD_pip
	{"pip5", pip5_register, 1},
	{"pip6", pip6_register, 1},
    #endif
    #ifdef ENABLE_CARD_smartcan
	{"smartcan", smartcan_register, 1},
    #endif
    #ifdef ENABLE_CARD_nsi
	{"nsican", nsi_register, 1},
    #endif
    #ifdef ENABLE_CARD_cc_can104
	{"cc104", cc104_register, 1},
    #endif
    #ifdef ENABLE_CARD_aim104
	{"aim104", aim104_register, 1},
    #endif
    #ifdef ENABLE_CARD_pc_i03
	{"pc-i03", pci03_register, 1},
    #endif
    #ifdef ENABLE_CARD_pcm3680
	{"pcm3680", pcm3680_register, 2},
    #endif
    #ifdef ENABLE_CARD_pccan
	{"pccan-f", pccanf_register, 1},
	{"pccan-s", pccanf_register, 1},
	{"pccan-d", pccand_register, 2},
	{"pccan-q", pccanq_register, 4},
    #endif
    #ifdef ENABLE_CARD_m437
	{"m437", m437_register, 1},
    #endif
    #ifdef ENABLE_CARD_pcccan
	{"pcccan", pcccan_register, 1},
    #endif
    #ifdef ENABLE_CARD_ssv
	{"ssv",	ssv_register, 2},
    #endif
    #ifdef ENABLE_CARD_bfadcan
	{"bfadcan", bfadcan_register, 1},
    #endif
    #ifdef ENABLE_CARD_pikronisa
	{"pikronisa", pikronisa_register, 1},
    #endif
	{NULL}
};

const struct boardtype_t* boardtype_find(const char *str)
{
	const struct boardtype_t *brp;

	for(brp=can_boardtypes;brp->boardtype;brp++) {
		if(!strcmp(str,brp->boardtype))
			return brp;
	}

	return NULL;
}
